@@ -5,8 +5,6 @@
  *
  * PHP version 5 and 7
  *
- * @category  Math
- * @package   BigInteger
  * @author    Jim Wigginton <terrafrost@php.net>
  * @copyright 2017 Jim Wigginton
  * @license   http://www.opensource.org/licenses/mit-license.html  MIT License
@@ -15,15 +13,13 @@
 
 namespace phpseclib3\Math\BigInteger\Engines;
 
-use ParagonIE\ConstantTime\Hex;
+use phpseclib3\Common\Functions\Strings;
 use phpseclib3\Exception\BadConfigurationException;
 
 /**
  * Pure-PHP Engine.
  *
- * @package PHP
  * @author  Jim Wigginton <terrafrost@php.net>
- * @access  public
  */
 abstract class PHP extends Engine
 {
@@ -33,8 +29,7 @@ abstract class PHP extends Engine
      * Rather than create a thousands and thousands of new BigInteger objects in repeated function calls to add() and
      * multiply() or whatever, we'll just work directly on arrays, taking them in as parameters and returning them.
      *
-     * @access protected
-    */
+     */
     /**
      * $result[self::VALUE] contains the value.
      */
@@ -50,7 +45,6 @@ abstract class PHP extends Engine
      *
      * At what point do we switch between Karatsuba multiplication and schoolbook long multiplication?
      *
-     * @access private
      */
     const KARATSUBA_CUTOFF = 25;
 
@@ -59,7 +53,6 @@ abstract class PHP extends Engine
      *
      * @see parent::bitwise_leftRotate()
      * @see parent::bitwise_rightRotate()
-     * @access protected
      */
     const FAST_BITWISE = true;
 
@@ -67,7 +60,6 @@ abstract class PHP extends Engine
      * Engine Directory
      *
      * @see parent::setModExpEngine
-     * @access protected
      */
     const ENGINE_DIR = 'PHP';
 
@@ -76,15 +68,15 @@ abstract class PHP extends Engine
      *
      * @param mixed $x integer Base-10 number or base-$base number if $base set.
      * @param int $base
+     * @return PHP
      * @see parent::__construct()
-     * @return \phpseclib3\Math\BigInteger\Engines\PHP
      */
     public function __construct($x = 0, $base = 10)
     {
-        if (!isset(static::$isValidEngine)) {
-            static::$isValidEngine = static::isValidEngine();
+        if (!isset(static::$isValidEngine[static::class])) {
+            static::$isValidEngine[static::class] = static::isValidEngine();
         }
-        if (!static::$isValidEngine) {
+        if (!static::$isValidEngine[static::class]) {
             throw new BadConfigurationException(static::class . ' is not setup correctly on this system');
         }
 
@@ -103,7 +95,7 @@ abstract class PHP extends Engine
         switch (abs($base)) {
             case 16:
                 $x = (strlen($this->value) & 1) ? '0' . $this->value : $this->value;
-                $temp = new static(Hex::decode($x), 256);
+                $temp = new static(Strings::hex2bin($x), 256);
                 $this->value = $temp->value;
                 break;
             case 10:
@@ -119,7 +111,12 @@ abstract class PHP extends Engine
                     $x = substr($x, 1);
                 }
 
-                $x = str_pad($x, strlen($x) + ((static::MAX10LEN - 1) * strlen($x)) % static::MAX10LEN, 0, STR_PAD_LEFT);
+                $x = str_pad(
+                    $x,
+                    strlen($x) + ((static::MAX10LEN - 1) * strlen($x)) % static::MAX10LEN,
+                    0,
+                    STR_PAD_LEFT
+                );
                 while (strlen($x)) {
                     $temp = $temp->multiply($multiplier);
                     $temp = $temp->add(new static($this->int2bytes(substr($x, 0, static::MAX10LEN)), 256));
@@ -165,7 +162,12 @@ abstract class PHP extends Engine
         $result = '';
         while (count($temp->value)) {
             list($temp, $mod) = $temp->divide($divisor);
-            $result = str_pad(isset($mod->value[0]) ? $mod->value[0] : '', static::MAX10LEN, '0', STR_PAD_LEFT) . $result;
+            $result = str_pad(
+                isset($mod->value[0]) ? $mod->value[0] : '',
+                static::MAX10LEN,
+                '0',
+                STR_PAD_LEFT
+            ) . $result;
         }
         $result = ltrim($result, '0');
         if (empty($result)) {
@@ -199,7 +201,12 @@ abstract class PHP extends Engine
         $result = implode('', array_map('chr', $result));
 
         return $this->precision > 0 ?
-            str_pad(substr($result, -(($this->precision + 7) >> 3)), ($this->precision + 7) >> 3, chr(0), STR_PAD_LEFT) :
+            str_pad(
+                substr($result, -(($this->precision + 7) >> 3)),
+                ($this->precision + 7) >> 3,
+                chr(0),
+                STR_PAD_LEFT
+            ) :
             $result;
     }
 
@@ -240,7 +247,7 @@ abstract class PHP extends Engine
 
             $temp = self::subtractHelper($x_value, false, $y_value, false);
             $temp[self::SIGN] = self::compareHelper($x_value, false, $y_value, false) > 0 ?
-                                          $x_negative : $y_negative;
+                $x_negative : $y_negative;
 
             return $temp;
         }
@@ -256,7 +263,7 @@ abstract class PHP extends Engine
         $value[count($value)] = 0; // just in case the carry adds an extra digit
 
         $carry = 0;
-        for ($i = 0, $j = 1; $j < $size; $i+=2, $j+=2) {
+        for ($i = 0, $j = 1; $j < $size; $i += 2, $j += 2) {
             //$sum = $x_value[$j] * static::BASE_FULL + $x_value[$i] + $y_value[$j] * static::BASE_FULL + $y_value[$i] + $carry;
             $sum = ($x_value[$j] + $y_value[$j]) * static::BASE_FULL + $x_value[$i] + $y_value[$i] + $carry;
             $carry = $sum >= static::MAX_DIGIT2; // eg. floor($sum / 2**52); only possible values (in any base) are 0 and 1
@@ -264,7 +271,7 @@ abstract class PHP extends Engine
 
             $temp = static::BASE === 26 ? intval($sum / 0x4000000) : ($sum >> 31);
 
-            $value[$i] = (int) ($sum - static::BASE_FULL * $temp); // eg. a faster alternative to fmod($sum, 0x4000000)
+            $value[$i] = (int)($sum - static::BASE_FULL * $temp); // eg. a faster alternative to fmod($sum, 0x4000000)
             $value[$j] = $temp;
         }
 
@@ -297,7 +304,7 @@ abstract class PHP extends Engine
      * @param bool $y_negative
      * @return array
      */
-    static function subtractHelper(array $x_value, $x_negative, array $y_value, $y_negative)
+    public static function subtractHelper(array $x_value, $x_negative, array $y_value, $y_negative)
     {
         $x_size = count($x_value);
         $y_size = count($y_value);
@@ -346,7 +353,7 @@ abstract class PHP extends Engine
         // at this point, $x_value should be at least as big as - if not bigger than - $y_value
 
         $carry = 0;
-        for ($i = 0, $j = 1; $j < $y_size; $i+=2, $j+=2) {
+        for ($i = 0, $j = 1; $j < $y_size; $i += 2, $j += 2) {
             $sum = ($x_value[$j] - $y_value[$j]) * static::BASE_FULL + $x_value[$i] - $y_value[$i] - $carry;
 
             $carry = $sum < 0; // eg. floor($sum / 2**52); only possible values (in any base) are 0 and 1
@@ -354,7 +361,7 @@ abstract class PHP extends Engine
 
             $temp = static::BASE === 26 ? intval($sum / 0x4000000) : ($sum >> 31);
 
-            $x_value[$i] = (int) ($sum - static::BASE_FULL * $temp);
+            $x_value[$i] = (int)($sum - static::BASE_FULL * $temp);
             $x_value[$j] = $temp;
         }
 
@@ -485,7 +492,7 @@ abstract class PHP extends Engine
         for ($j = 0; $j < $x_length; ++$j) { // ie. $i = 0
             $temp = $x_value[$j] * $y_value[0] + $carry; // $product_value[$k] == 0
             $carry = static::BASE === 26 ? intval($temp / 0x4000000) : ($temp >> 31);
-            $product_value[$j] = (int) ($temp - static::BASE_FULL * $carry);
+            $product_value[$j] = (int)($temp - static::BASE_FULL * $carry);
         }
 
         $product_value[$j] = $carry;
@@ -498,7 +505,7 @@ abstract class PHP extends Engine
             for ($j = 0, $k = $i; $j < $x_length; ++$j, ++$k) {
                 $temp = $product_value[$k] + $x_value[$j] * $y_value[$i] + $carry;
                 $carry = static::BASE === 26 ? intval($temp / 0x4000000) : ($temp >> 31);
-                $product_value[$k] = (int) ($temp - static::BASE_FULL * $carry);
+                $product_value[$k] = (int)($temp - static::BASE_FULL * $carry);
             }
 
             $product_value[$k] = $carry;
@@ -515,9 +522,9 @@ abstract class PHP extends Engine
      * same.  If the remainder would be negative, the "common residue" is equal to the sum of the remainder
      * and the divisor (basically, the "common residue" is the first positive modulo).
      *
-     * @param \phpseclib3\Math\BigInteger\engines\PHP $y
-     * @return array
-     * @internal This function is based off of {@link http://www.cacr.math.uwaterloo.ca/hac/about/chap14.pdf#page=9 HAC 14.20}.
+     * @return array{static, static}
+     * @internal This function is based off of
+     *     {@link http://www.cacr.math.uwaterloo.ca/hac/about/chap14.pdf#page=9 HAC 14.20}.
      */
     protected function divideHelper(PHP $y)
     {
@@ -545,7 +552,7 @@ abstract class PHP extends Engine
             $temp = new static();
             $temp->value = [1];
             $temp->is_negative = $x_sign != $y_sign;
-            return [$this->normalize($temp), $this->normalize(static::$zero)];
+            return [$this->normalize($temp), $this->normalize(static::$zero[static::class])];
         }
 
         if ($diff < 0) {
@@ -553,7 +560,7 @@ abstract class PHP extends Engine
             if ($x_sign) {
                 $x = $y->subtract($x);
             }
-            return [$this->normalize(static::$zero), $this->normalize($x)];
+            return [$this->normalize(static::$zero[static::class]), $this->normalize($x)];
         }
 
         // normalize $x and $y as described in HAC 14.23 / 14.24
@@ -575,13 +582,13 @@ abstract class PHP extends Engine
         static $temp, $lhs, $rhs;
         if (!isset($temp)) {
             $temp = new static();
-            $lhs =  new static();
-            $rhs =  new static();
+            $lhs = new static();
+            $rhs = new static();
         }
-	if (static::class != get_class($temp)) {
+        if (static::class != get_class($temp)) {
             $temp = new static();
-            $lhs =  new static();
-            $rhs =  new static();
+            $lhs = new static();
+            $rhs = new static();
         }
         $temp_value = &$temp->value;
         $rhs_value =  &$rhs->value;
@@ -642,7 +649,7 @@ abstract class PHP extends Engine
 
             $x = $x->subtract($temp);
 
-            if ($x->compare(static::$zero) < 0) {
+            if ($x->compare(static::$zero[static::class]) < 0) {
                 $temp_value = array_merge($adjust, $y_value);
                 $x = $x->add($temp);
 
@@ -683,7 +690,7 @@ abstract class PHP extends Engine
         for ($i = count($dividend) - 1; $i >= 0; --$i) {
             $temp = static::BASE_FULL * $carry + $dividend[$i];
             $result[$i] = self::safe_divide($temp, $divisor);
-            $carry = (int) ($temp - $divisor * $result[$i]);
+            $carry = (int)($temp - $divisor * $result[$i]);
         }
 
         return [$result, $carry];
@@ -704,18 +711,19 @@ abstract class PHP extends Engine
     private static function safe_divide($x, $y)
     {
         if (static::BASE === 26) {
-            return (int) ($x / $y);
+            return (int)($x / $y);
         }
 
         // static::BASE === 31
+        /** @var int */
         return ($x - ($x % $y)) / $y;
     }
 
-    /*
+    /**
      * Convert an array / boolean to a PHP BigInteger object
      *
      * @param array $arr
-     * @return \phpseclib3\Math\BigInteger\Engines\PHP
+     * @return static
      */
     protected function convertToObj(array $arr)
     {
@@ -732,7 +740,7 @@ abstract class PHP extends Engine
      * Removes leading zeros and truncates (if necessary) to maintain the appropriate precision
      *
      * @param PHP $result
-     * @return PHP
+     * @return static
      */
     protected function normalize(PHP $result)
     {
@@ -755,12 +763,14 @@ abstract class PHP extends Engine
             for ($i = 0; $i < $length; ++$i) {
                 $value[$i] = $value[$i] & $result->bitmask->value[$i];
             }
+
+            $value = static::trim($value);
         }
 
         return $result;
     }
 
-    /*
+    /**
      * Compares two numbers.
      *
      * @param array $x_value
@@ -798,7 +808,7 @@ abstract class PHP extends Engine
     /**
      * Absolute value.
      *
-     * @return \phpseclib3\Math\BigInteger\Engines\PHP
+     * @return PHP
      */
     public function abs()
     {
@@ -813,8 +823,8 @@ abstract class PHP extends Engine
      *
      * Removes leading zeros
      *
-     * @param array $value
-     * @return PHP
+     * @param list<static> $value
+     * @return list<static>
      */
     protected static function trim(array $value)
     {
@@ -834,7 +844,7 @@ abstract class PHP extends Engine
      * Shifts BigInteger's by $shift bits, effectively dividing by 2**$shift.
      *
      * @param int $shift
-     * @return \phpseclib3\Math\BigInteger\Engines\PHP
+     * @return PHP
      */
     public function bitwise_rightShift($shift)
     {
@@ -854,7 +864,7 @@ abstract class PHP extends Engine
      * Shifts BigInteger's by $shift bits, effectively multiplying by 2**$shift.
      *
      * @param int $shift
-     * @return \phpseclib3\Math\BigInteger\Engines\PHP
+     * @return PHP
      */
     public function bitwise_leftShift($shift)
     {
@@ -886,7 +896,7 @@ abstract class PHP extends Engine
      * @return array
      */
     protected static function array_repeat($input, $multiplier)
-    { 
+    {
         return $multiplier ? array_fill(0, $multiplier, $input) : [];
     }
 
@@ -903,7 +913,7 @@ abstract class PHP extends Engine
             return;
         }
 
-        $num_digits = (int) ($shift / static::BASE);
+        $num_digits = (int)($shift / static::BASE);
         $shift %= static::BASE;
         $shift = 1 << $shift;
 
@@ -912,7 +922,7 @@ abstract class PHP extends Engine
         for ($i = 0; $i < count($this->value); ++$i) {
             $temp = $this->value[$i] * $shift + $carry;
             $carry = static::BASE === 26 ? intval($temp / 0x4000000) : ($temp >> 31);
-            $this->value[$i] = (int) ($temp - $carry * static::BASE_FULL);
+            $this->value[$i] = (int)($temp - $carry * static::BASE_FULL);
         }
 
         if ($carry) {
@@ -937,7 +947,7 @@ abstract class PHP extends Engine
             return;
         }
 
-        $num_digits = (int) ($shift / static::BASE);
+        $num_digits = (int)($shift / static::BASE);
         $shift %= static::BASE;
         $carry_shift = static::BASE - $shift;
         $carry_mask = (1 << $shift) - 1;
@@ -967,7 +977,7 @@ abstract class PHP extends Engine
     protected function powModInner(PHP $e, PHP $n)
     {
         try {
-            $class = static::$modexpEngine;
+            $class = static::$modexpEngine[static::class];
             return $class::powModHelper($this, $e, $n, static::class);
         } catch (\Exception $err) {
             return PHP\DefaultEngine::powModHelper($this, $e, $n, static::class);
@@ -977,8 +987,8 @@ abstract class PHP extends Engine
     /**
      * Performs squaring
      *
-     * @param array $x
-     * @return array
+     * @param list<static> $x
+     * @return list<static>
      */
     protected static function square(array $x)
     {
@@ -1009,13 +1019,13 @@ abstract class PHP extends Engine
 
             $temp = $square_value[$i2] + $value[$i] * $value[$i];
             $carry = static::BASE === 26 ? intval($temp / 0x4000000) : ($temp >> 31);
-            $square_value[$i2] = (int) ($temp - static::BASE_FULL * $carry);
+            $square_value[$i2] = (int)($temp - static::BASE_FULL * $carry);
 
             // note how we start from $i+1 instead of 0 as we do in multiplication.
             for ($j = $i + 1, $k = $i2 + 1; $j <= $max_index; ++$j, ++$k) {
                 $temp = $square_value[$k] + 2 * $value[$j] * $value[$i] + $carry;
                 $carry = static::BASE === 26 ? intval($temp / 0x4000000) : ($temp >> 31);
-                $square_value[$k] = (int) ($temp - static::BASE_FULL * $carry);
+                $square_value[$k] = (int)($temp - static::BASE_FULL * $carry);
             }
 
             // the following line can yield values larger 2**15.  at this point, PHP should switch
@@ -1093,7 +1103,7 @@ abstract class PHP extends Engine
         }
 
         $value = $this->value;
-        foreach (static::$primes as $prime) {
+        foreach (static::PRIMES as $prime) {
             list(, $r) = self::divide_digit($value, $prime);
             if (!$r) {
                 return count($value) == 1 && $value[0] == $prime;
@@ -1108,9 +1118,9 @@ abstract class PHP extends Engine
      *
      * ie. $s = gmp_scan1($n, 0) and $r = gmp_div_q($n, gmp_pow(gmp_init('2'), $s));
      *
-     * @see self::isPrime()
      * @param PHP $r
      * @return int
+     * @see self::isPrime()
      */
     public static function scan1divide(PHP $r)
     {
@@ -1136,14 +1146,14 @@ abstract class PHP extends Engine
      */
     protected function powHelper(PHP $n)
     {
-        if ($n->compare(static::$zero) == 0) {
+        if ($n->compare(static::$zero[static::class]) == 0) {
             return new static(1);
         } // n^0 = 1
 
         $temp = clone $this;
-        while (!$n->equals(static::$one)) {
+        while (!$n->equals(static::$one[static::class])) {
             $temp = $temp->multiply($this);
-            $n = $n->subtract(static::$one);
+            $n = $n->subtract(static::$one[static::class]);
         }
 
         return $temp;
@@ -1152,34 +1162,34 @@ abstract class PHP extends Engine
     /**
      * Is Odd?
      *
-     * @return boolean
+     * @return bool
      */
     public function isOdd()
     {
-        return (bool) ($this->value[0] & 1);
+        return (bool)($this->value[0] & 1);
     }
 
     /**
      * Tests if a bit is set
      *
-     * @return boolean
+     * @return bool
      */
     public function testBit($x)
     {
-        $digit = floor($x / static::BASE);
+        $digit = (int) floor($x / static::BASE);
         $bit = $x % static::BASE;
 
         if (!isset($this->value[$digit])) {
             return false;
         }
 
-        return (bool) ($this->value[$digit] & (1 << $bit));
+        return (bool)($this->value[$digit] & (1 << $bit));
     }
 
     /**
      * Is Negative?
      *
-     * @return boolean
+     * @return bool
      */
     public function isNegative()
     {
@@ -1191,7 +1201,7 @@ abstract class PHP extends Engine
      *
      * Given $k, returns -$k
      *
-     * @return BigInteger
+     * @return static
      */
     public function negate()
     {
@@ -1207,7 +1217,7 @@ abstract class PHP extends Engine
      * Splits BigInteger's into chunks of $split bits
      *
      * @param int $split
-     * @return \phpseclib3\Math\BigInteger\Engines\PHP[]
+     * @return list<static>
      */
     public function bitwise_split($split)
     {
@@ -1215,7 +1225,7 @@ abstract class PHP extends Engine
             throw new \RuntimeException('Offset must be greater than 1');
         }
 
-        $width = (int) ($split / static::BASE);
+        $width = (int)($split / static::BASE);
         if (!$width) {
             $arr = $this->bitwise_small_split($split);
             return array_map(function ($digit) {
@@ -1234,7 +1244,7 @@ abstract class PHP extends Engine
             $digit = [];
             if (!$overflow) {
                 $digit = array_slice($val, $i, $width);
-                $i+= $width;
+                $i += $width;
                 $overflow = $split % static::BASE;
                 if ($overflow) {
                     $mask = (1 << $overflow) - 1;
@@ -1244,9 +1254,9 @@ abstract class PHP extends Engine
             } else {
                 $remaining = static::BASE - $overflow;
                 $tempsplit = $split - $remaining;
-                $tempwidth = (int) ($tempsplit / static::BASE + 1);
+                $tempwidth = (int)($tempsplit / static::BASE + 1);
                 $digit = array_slice($val, $i, $tempwidth);
-                $i+= $tempwidth;
+                $i += $tempwidth;
                 $tempoverflow = $tempsplit % static::BASE;
                 if ($tempoverflow) {
                     $tempmask = (1 << $tempoverflow) - 1;
@@ -1274,7 +1284,7 @@ abstract class PHP extends Engine
      * Bitwise Split where $split < static::BASE
      *
      * @param int $split
-     * @return \phpseclib3\Math\BigInteger\Engines\PHP[]
+     * @return list<int>
      */
     private function bitwise_small_split($split)
     {
@@ -1289,9 +1299,9 @@ abstract class PHP extends Engine
         $remaining = static::BASE;
         while ($i != $len) {
             $digit = $val[$i] & $mask;
-            $val[$i]>>= $split;
+            $val[$i] >>= $split;
             if (!$overflow) {
-                $remaining-= $split;
+                $remaining -= $split;
                 $overflow = $split <= $remaining ? 0 : $split - $remaining;
 
                 if (!$remaining) {
@@ -1299,10 +1309,10 @@ abstract class PHP extends Engine
                     $remaining = static::BASE;
                     $overflow = 0;
                 }
-            } else if (++$i != $len) {
+            } elseif (++$i != $len) {
                 $tempmask = (1 << $overflow) - 1;
-                $digit|= ($val[$i] & $tempmask) << $remaining;
-                $val[$i]>>= $overflow;
+                $digit |= ($val[$i] & $tempmask) << $remaining;
+                $val[$i] >>= $overflow;
                 $remaining = static::BASE - $overflow;
                 $overflow = $split <= $remaining ? 0 : $split - $remaining;
             }
@@ -1316,4 +1326,4 @@ abstract class PHP extends Engine
 
         return array_reverse($vals);
     }
-}
\ No newline at end of file
+}
