@@ -1,17 +1,20 @@
 <?php
+
 /**
  * @author    Andreas Fischer <bantu@phpbb.com>
  * @copyright 2013 Andreas Fischer
  * @license   http://www.opensource.org/licenses/mit-license.html  MIT License
  */
 
+namespace phpseclib3\Tests\Unit\Crypt\AES;
+
 use phpseclib3\Crypt\AES;
-use phpseclib3\Crypt\Common\BlockCipher;
 use phpseclib3\Crypt\Rijndael;
 use phpseclib3\Exception\InconsistentSetupException;
 use phpseclib3\Exception\InsufficientSetupException;
+use phpseclib3\Tests\PhpseclibTestCase;
 
-abstract class Unit_Crypt_AES_TestCase extends PhpseclibTestCase
+abstract class TestCase extends PhpseclibTestCase
 {
     protected $engine;
 
@@ -128,7 +131,7 @@ abstract class Unit_Crypt_AES_TestCase extends PhpseclibTestCase
     /**
      * Produces all combinations of test values.
      *
-     * @return array
+     * @return list<array{string, string, array}>
      */
     public function continuousBufferBatteryCombos()
     {
@@ -170,6 +173,16 @@ abstract class Unit_Crypt_AES_TestCase extends PhpseclibTestCase
         return $result;
     }
 
+    /**
+     * @return array<array{string, string, array}>
+     */
+    public function continuousBufferBatteryCombosWithoutSingleCombos()
+    {
+        return array_filter($this->continuousBufferBatteryCombos(), function (array $continuousBufferBatteryCombo) {
+            return count($continuousBufferBatteryCombo[2]) > 1;
+        });
+    }
+
     /**
      * @dataProvider continuousBufferBatteryCombos
      */
@@ -189,7 +202,7 @@ abstract class Unit_Crypt_AES_TestCase extends PhpseclibTestCase
         $result = '';
         foreach ($test as $len) {
             $temp = str_repeat('d', $len);
-            $str.= $temp;
+            $str .= $temp;
         }
 
         $c1 = $aes->$op($str);
@@ -200,14 +213,12 @@ abstract class Unit_Crypt_AES_TestCase extends PhpseclibTestCase
         $aes->setKey($key);
         $aes->setIV($iv);
 
-        if (!$this->_checkEngine($aes)) {
-            return;
-        }
+        $this->_checkEngine($aes);
 
         foreach ($test as $len) {
             $temp = str_repeat('d', $len);
             $output = $aes->$op($temp);
-            $result.= $output;
+            $result .= $output;
         }
 
         $c2 = $result;
@@ -218,14 +229,10 @@ abstract class Unit_Crypt_AES_TestCase extends PhpseclibTestCase
     /**
      * Pretty much the same as testContinuousBufferBattery with the caveat that continuous mode is not enabled.
      *
-     * @dataProvider continuousBufferBatteryCombos
+     * @dataProvider continuousBufferBatteryCombosWithoutSingleCombos
      */
     public function testNonContinuousBufferBattery($op, $mode, $test)
     {
-        if (count($test) == 1) {
-            return;
-        }
-
         $iv = str_repeat('x', 16);
         $key = str_repeat('a', 16);
 
@@ -240,7 +247,7 @@ abstract class Unit_Crypt_AES_TestCase extends PhpseclibTestCase
         $result = '';
         foreach ($test as $len) {
             $temp = str_repeat('d', $len);
-            $str.= $temp;
+            $str .= $temp;
         }
 
         $c1 = $aes->$op($str);
@@ -255,7 +262,7 @@ abstract class Unit_Crypt_AES_TestCase extends PhpseclibTestCase
         foreach ($test as $len) {
             $temp = str_repeat('d', $len);
             $output = $aes->$op($temp);
-            $result.= $output;
+            $result .= $output;
         }
 
         $c2 = $result;
