@@ -19,9 +19,9 @@ class BigIntegerTest extends PhpseclibTestCase
     {
         eval(<<<ENGINE
 namespace phpseclib3\Math\BigInteger\Engines;
-class ${className} extends \phpseclib3\Math\BigInteger\Engines\Engine {
+class $className extends \phpseclib3\Math\BigInteger\Engines\Engine {
 	public function __construct(){} 
-	public static function isValidEngine() { return ${isValid}; }
+	public static function isValidEngine() { return $isValid; }
 	public static function setModExpEngine(\$engine){} 
 	public function toString() { return __CLASS__; }
 }
